package com.fs.his.service;

import java.util.List;
import com.fs.his.domain.FsDoctorPrescribe;
import com.fs.his.param.FsDoctorPrescribeListDParam;
import com.fs.his.param.FsDoctorPrescribeVO;
import com.fs.his.vo.FsDoctorPrescribeListDVO;

/**
 * 医生处方Service接口
 *
 * @author fs
 * @date 2023-08-08
 */
public interface IFsDoctorPrescribeService
{
    /**
     * 查询医生处方
     *
     * @param prescribeId 医生处方主键
     * @return 医生处方
     */
    public FsDoctorPrescribe selectFsDoctorPrescribeByPrescribeId(Long prescribeId);

    /**
     * 查询医生处方列表
     *
     * @param fsDoctorPrescribe 医生处方
     * @return 医生处方集合
     */
    public List<FsDoctorPrescribe> selectFsDoctorPrescribeList(FsDoctorPrescribe fsDoctorPrescribe);

    /**
     * 新增医生处方
     *
     * @param fsDoctorPrescribe 医生处方
     * @return 结果
     */
    public int insertFsDoctorPrescribe(FsDoctorPrescribe fsDoctorPrescribe);

    /**
     * 修改医生处方
     *
     * @param fsDoctorPrescribe 医生处方
     * @return 结果
     */
    public int updateFsDoctorPrescribe(FsDoctorPrescribe fsDoctorPrescribe);

    /**
     * 批量删除医生处方
     *
     * @param prescribeIds 需要删除的医生处方主键集合
     * @return 结果
     */
    public int deleteFsDoctorPrescribeByPrescribeIds(Long[] prescribeIds);

    /**
     * 删除医生处方信息
     *
     * @param prescribeId 医生处方主键
     * @return 结果
     */
    public int deleteFsDoctorPrescribeByPrescribeId(Long prescribeId);

    List<FsDoctorPrescribeListDVO> selectFsDoctorPrescribeListDVO(FsDoctorPrescribeListDParam param);

    List<FsDoctorPrescribeVO> selectFsDoctorPrescribeListVO(FsDoctorPrescribe fsDoctorPrescribe);
}
